@@ -1,16 +1,15 @@
-from syntok.tokenizer import Tokenizer
+import spacy
 
-def tokenize_by_space(text):
-    tokens = text.split(" ")
-    tokens = [t for t in tokens if len(t) > 0]
-    return tokens
 
 class GazetteerTermExtractor:
     def __init__(self, term_list):
-        self.tokenizer = Tokenizer()
+        spacy_model = "en_core_web_sm"
+        if not spacy.util.is_package(spacy_model):
+            spacy.cli.download(spacy_model)
+        self.en_model = spacy.load(spacy_model, disable=['ner', 'tagger', 'parser'])
         self.terms_by_lengths = {}
         for term in term_list:
-            term = tuple(tokenize_by_space(term))
+            term = tuple(self._spacy_tokenize(term))
             length = len(term)
             if length not in self.terms_by_lengths:
                 self.terms_by_lengths[length] = set()
@@ -18,16 +17,15 @@ class GazetteerTermExtractor:
         self.window_sizes = sorted(list(self.terms_by_lengths.keys()))
 
     def extract_terms(self, text: str):
-        sentence = list(self.tokenizer.tokenize(text))
-        tokens = [t.value for t in sentence]
+        sentence = self._spacy_tokenize_spans(text)
+        tokens = [text[s:e] for s, e in sentence]
 
         found_terms = self._find_terms(tokens)
         unique_dnts = self._resolve_overlappings(found_terms, len(tokens))
 
         result = []
         for start, end in unique_dnts:
-            start_char = sentence[start].offset
-            end_char = sentence[end - 1].offset + len(tokens[end - 1])
+            start_char, end_char = sentence[start][0], sentence[end - 1][1]
             result.append((start_char, end_char))
         return result
 
@@ -62,24 +60,37 @@ class GazetteerTermExtractor:
         unique_terms = sorted(list(set(unique_terms)), key=lambda d: d[0])
         return unique_terms
 
+    def _spacy_tokenize(self, text):
+        tokens = self.en_model(text)
+        return [str(t) for t in tokens]
+
+
+    def _spacy_tokenize_spans(self, text):
+        tokens = self.en_model(text)
+        return [(t.idx, t.idx + len(str(t))) for t in tokens if len(str(t).strip()) > 0]
+
 
 def test():
     extractor = GazetteerTermExtractor([
-        "Azure", "Microsoft", "AI for LOC", "Power BI", "Xbox", "Xbox 360", "Xbox One X", "Xbox Series X", "AppSource"
+        "Azure", "Microsoft", "AI for LOC", "Power BI", "Xbox", "Xbox 360", "Xbox One X", "Xbox Series X", "AppSource", "SharePoint", "Azure DevOps"
     ])
 
-    for text in [
-        "I don't know how to use Azure",
-        "Microsoft is my favourite OS",
-        "I am working on new project called AI  for LOC",
-        "Power BI directional graph",
-        "Xbox 360 is better than Xbox One, but worse than Xbox Series X!",
-        "I love App Source"
+    for text, terms in [
+        ("I don't know how to use Azure", ["Azure"]),
+        ("Microsoft is my favourite OS", ["Microsoft"]),
+        ("I am working on new project called AI  for LOC", ["AI  for LOC"]),
+        ("Power BI directional graph", ["Power BI"]),
+        ("Xbox 360 is better than Xbox One, but worse than Xbox Series X!", ["Xbox 360", "Xbox", "Xbox Series X"]),
+        ("I love App Source", []),
+        ("SharePoint scenarios overview", ["SharePoint"]),
+        ("Azure DevOps project must be used to store and manage​ your test cases, test plans, and test case results.", ["Azure DevOps"])
     ]:
-        print(text)
-        for s, e in extractor.extract_terms(text):
-            print(text[s:e])
-        print()
+        extracted_terms = [text[s:e] for s, e in extractor.extract_terms(text)]
+        if terms != extracted_terms:
+            print(text)
+            print("Expected terms: %s" % terms)
+            print("Actual terms: %s" % extracted_terms)
+            print()
 
 
 if __name__ == "__main__":
